import glob
import os
import re
import shutil
import subprocess
import threading
import typing

from PIL import ImageGrab

import Socket

Soc = typing.Union[Socket.socket.socket, Socket.Socket]


def exit(client_socket: Soc):
    """disconnect the client and server."""
    return True  # functions can return true to disconnect from the client


def screen(client_socket: Soc):
    """sends a screenshot of the server to the client."""
    # TODO: change 'send_sized' to 'send_chopped' and make sure it works
    im = ImageGrab.grab()
    im.save(r'sent_image.png')
    with open(r'sent_image.png', 'rb') as image:
        client_socket.send_chopped(image.read())


def content(client_socket: Soc, directory):
    """sends a list of file names, separated by comas, from the specified directory.gets the directory."""
    if os.path.isdir(directory):
        dirs = glob.glob(f'{directory}/*.*')
        files = []
        for dir in dirs[:]:
            files.append(re.split('(\\\\)|(/)', dir)[-1])
        client_socket.send_sized(',\n'.join(files))
    else:
        client_socket.send_sized(f'the path {directory} is not a directory or does not exist')


def remove(client_socket: Soc, path):
    """delete a specific file. gets the file's path."""
    try:
        os.remove(path)
    except FileNotFoundError:
        client_socket.send_sized(f'the file {path} does not exist')
    except PermissionError:
        client_socket.send_sized(f'the path {path} is not a file')
    else:
        client_socket.send_sized('deleted')


def move(client_socket: Soc, source, target):
    """copy a specific file to a directory. gets the directory of the file and the directory to copy the file to"""
    if not os.path.isfile(source):
        client_socket.send_sized(f"the path '{source}' is not a file or does not exist")
    elif not os.path.isdir(target):
        client_socket.send_sized(f"the path '{target}' is not a directory or does not exist")
    else:
        shutil.move(source, target)
        client_socket.send_sized(f'moved {source} to {target}')


def run(client_socket: Soc, to_run):
    """runs a specific program. gets the program's name"""
    if shutil.which(to_run):
        threading.Thread(target=subprocess.call, args=(to_run,)).start()
        client_socket.send_sized(f'running {to_run}')
    else:
        client_socket.send_sized(f'the program {to_run} does not exist')


def help(client_socket: Soc):
    """sends a list of commands and their documentation to the client"""
    docs = ['some commands may have parameters, they shall be specified as follows:\n'
            'command-name param1 param2 ...\n']
    for name, method in globals().items():
        # anything that isn't a function has to be filtered out
        # together with helper functions (start with _) and 'Soc'
        if callable(method) and name[0] != '_' and name != 'Soc':
            docs.append(f'{name.upper()}: {method.__doc__}\n')
    client_socket.send_sized(''.join(docs))
